/*
PRICE was written by Graham Ruby.
This file is part of PRICE.

PRICE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PRICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PRICE.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef NUCLEICACIDUTILITIES_H
#define NUCLEICACIDUTILITIES_H

#include <stdexcept>
#include <string>
#include <limits>
#include <cmath>
#include <iostream>
#include <set>
#include <map>
#include "AssemblyException.h"
#include "ScoredSeq.h"
using namespace::std;

class nucleicAcidUtilities {

 public:

  // function defs
  static char reverseCompChar(char n);
  static string reverseComplement(string seq);
  // includes the null final char at the end
  static char* reverseComplement(char* seq, long length);



  /* KMER TOOLS
   * these are special tools/fields for dealing with kmers. note that,
   * for kmers, "N" will not be considered a valid nucleotide identity.
   */

  // i use the natural log function to calculate the base-5 log of INT_MAX
  static void figureMaxKmerLong();

  static int getMaxKmerSize();
  static long getMaxKmerLong();

  static void seqToSparseKmerLongs(set<long> * kmerSet, string * seq, int kmerSize);

  // an efficiency extension of the function below
  // first nucs are at the low end; 0 -> empty string
  // base-4: 1=A, 2=C, 3=G, 4=T (Ns are not valid kmer components)
  static void seqToKmerLongs(set<long> * kmerSet, string * seq, int kmerSize);

  // a version of the function above that keeps track of how many times each kmer appeared
  // in the sequence
  static void seqToCountedKmerLongs(map<long,long> * kmerSet, string * seq, int kmerSize, long edgeDistance);

  // first nucs are at the low end; 0 -> empty string
  // base-4: 1=A, 2=C, 3=G, 4=T (Ns are not valid kmer components)
  static long kmerToLong(string kmer);
  static string longToKmer(long longRep);

  // basic string interpretation
  static char* interpretString(char* rawSeq, long rawLen, set<char>* okToSkip);

  // LZ stuff
  static long calculateLzComplexity(ScoredSeq* seq);

 private:
  static long alphabetSize;
  static int maxKmerSize;
  static long maxKmerLong;
  static char numToChar[5];

  class LzTreeNode{
  public:
    LzTreeNode();
    ~LzTreeNode();
    void deepDelete();

    // returns NULL pointer (0) if there is no node
    LzTreeNode* getBranch(char nuc);
    void addBranch(char nuc);
  private:
    LzTreeNode* _branches[4];
  };

};


#endif
